==> I2P-Messenger_0.2.17_Beta_src.zip: I2P-Messenger/src/UserManager.cpp <==
/*
    <one line to give the program's name and a brief idea of what it does.>
    Copyright (C) <year>  <name of author>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

*/

#include "UserManager.h"

#include "Protocol.h"
#include "UserBlockManager.h"
#include "Core.h"


CUserManager::CUserManager(CCore& Core, QString UserFileWithPath) 
:mCore(Core),mUserFileWithPath(UserFileWithPath)
{
  
}

CUserManager::~CUserManager()
{
      for(int i=0;i<this->mUsers.count();i++){
	    delete mUsers.at(i);
      }
      
      mUsers.clear();
}

void CUserManager::loadUserList(){
	QFile file(mUserFileWithPath);
     	if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
		return;
	
	QString NickName;
	QString I2PDest;
	
	QString line;
	QStringList temp;

	QByteArray bUserList=file.readAll();
        QTextStream in(bUserList);
	in.skipWhiteSpace();
	
     	while (!in.atEnd()) {
		line = in.readLine(550);
		temp=line.split("\t");
	
		if(temp[0]=="Nick:"){
			NickName=temp[1];
			}
		else if(temp[0]=="I2PDest:"){
			I2PDest=temp[1];
			this->addNewUser(NickName,I2PDest,false);
		}
		else if(temp[0]=="Invisible:"){
			if(temp[1]=="true"){
				getUserByI2P_Destination(I2PDest)->setInvisible(true);
			}
		}
		else if(temp[0]=="TorDest:"){
			//ignore it
		}
	}
	file.close();	
}

void CUserManager::saveUserList()const{
     QFile file(QApplication::applicationDirPath()+"/users.config");
     file.open(QIODevice::WriteOnly | QIODevice::Text);
     QTextStream out(&file);
     QString InvisibleText;

	for(int i=0;i<this->mUsers.count();i++){
		if(mUsers.at(i)->getIsInvisible()==true){
			InvisibleText="true";
		}
		else{
			InvisibleText="false";
		}

		out<<"Nick:\t"<<(mUsers.at(i)->getName())<<endl
		   <<"I2PDest:\t"<<(mUsers.at(i)->getI2PDestination())<<endl
		   <<"Invisible:\t"<<InvisibleText<<endl;
	}
	out.flush();
	file.close();
}

CUser* CUserManager::getUserByI2P_ID(qint32 ID)const{

	for(int i=0;i<mUsers.size();i++){
		if(mUsers.at(i)->getI2PStreamID()==ID){
			
			return mUsers.at(i);
		}
	}
	
	return NULL;
}
CUser* CUserManager::getUserByI2P_Destination(QString Destination)const{
	for(int i=0;i<mUsers.size();i++){
		if(mUsers.at(i)->getI2PDestination()==Destination){
			return mUsers.at(i);
		}
	}
	
	return NULL;
}


QString CUserManager::getUserInfosByI2P_Destination(QString Destination)const
{
	QString Infos="No Informations recived";

	for(int i=0;i<mUsers.size();i++){
		if(mUsers.at(i)->getI2PDestination()==Destination){
			CUser* theUser=mUsers.at(i);

			Infos= "Nickname:\t\t"  		+theUser->getName()+"\n";
			Infos+="Clientname:\t\t"		+theUser->getClientName()+"\n";
			Infos+="Clientversion:\t"		+theUser->getClientVersion()+"\n";
			Infos+="Protocolversion:\t"		+theUser->getProtocolVersion()+"\n";
			Infos+="Protocolversion min (Filetransfer):"+theUser->getMinProtocolVersionFiletransfer()+"\n";
			Infos+="Protocolversion max (Filetransfer):"+theUser->getMaxProtocolVersionFiletransfer()+"\n";
			
			if(theUser->getProtocolVersion_D()>=0.3){
				CRecivedInfos recivedInfos=theUser->getRecivedUserInfos();
				QString sAge;
				sAge.setNum(recivedInfos.Age,10);

				Infos+="\nRecived Userinfos:\n";
				Infos+="Nickname:\t\t"		+recivedInfos.Nickname+"\n";
				Infos+="Gender:\t\t"		+recivedInfos.Gender+"\n";
				Infos+="Age:\t\t"		+sAge+"\n";
				Infos+="Interests:\t\t"		+recivedInfos.Interests;
			}
		}
	}
	return Infos;
}

const QList<CUser*> CUserManager::getUserList()const{
	return mUsers;
}


bool CUserManager::addNewUser(QString Name,QString I2PDestination,qint32 I2PStream_ID,bool SaveUserList){
	CUserBlockManager& UserBlockManager=*(mCore.getUserBlockManager());
	CProtocol& Protocol=*(mCore.getProtocol());
      //The 52 character (or 60 with .b32.i2p 
      
      
	if(I2PDestination.length() != 516 && I2PDestination.length() != 60){
		qCritical()<<"CCore::addNewUser"
			   <<"\n\tDestination length must be 516(for B64) or 60(for B32)"
			   <<"\n\tlength : "<<I2PDestination.length()
			   <<"\n\taction add new User ignored";
		return false;
	}
	
	if(I2PDestination.length() == 516){
		if(!I2PDestination.right(4).contains("AAAA",Qt::CaseInsensitive)){
			qCritical()<<"\nCCore::addNewUser"
				  <<"\n\tthe last 4 char must be AAAA"
				  <<"\n\tDestination:\n"<<I2PDestination
				  <<"\n\taction add new User ignored";
			return false;
		}
	}else{
		if(!I2PDestination.right(8).contains(".b32.i2p",Qt::CaseInsensitive)){
			qCritical()<<"\nCCore::addNewUser"
				  <<"\n\tthe last 8 char must be .b32.i2p"
				  <<"\n\tDestination:\n"<<I2PDestination
				  <<"\n\taction add new User ignored";
			return false;
		}
	}
	
	

	if(UserBlockManager.isDestinationInBlockList(I2PDestination)==true){
		qCritical()<<"\nCCore::addNewUser"
			   <<"\n\tThe Destination: "<<I2PDestination
			   <<"\n\tis on the blocklist"
			   <<"\n\taction add new User ignored";
			   
		return false;
	}

	
	if(this->checkIfUserExitsByI2PDestination(I2PDestination)==true){
		qCritical()<<"\nCCore::addNewUser"
		    <<"\n\tthere is allready a user with this destination"
		    <<"\n\taction add new User ignored";

		return false;
	}

	if(I2PDestination==mCore.getMyDestination()){
		qCritical()<<"\nCCore::addNewUser"
		<<"\n\tIt's not allowed to add your self"
		<<"\n\taction add new User ignored";

		return false;
	}
	

	//add newuser
	CSoundManager& SoundManager=*(mCore.getSoundManager());
	CUser* newuser=new CUser(mCore, Protocol,Name,I2PDestination,I2PStream_ID);
	connect(newuser,SIGNAL(signNewIncomingMessageRecived()),&SoundManager,
		SLOT(slotNewChatMessage()));

	connect(newuser,SIGNAL(signConnectionOnline()),&SoundManager,
		SLOT(slotUserGoOnline()));

	connect(newuser,SIGNAL(signConnectionOffline()),&SoundManager,
		SLOT(slotUserGoOffline()));

	connect(newuser,SIGNAL(signOnlineStateChanged()),this,
		SIGNAL(signUserStatusChanged()));


	this->mUsers.append(newuser);
	if(SaveUserList==true){
		saveUserList();
	}

	if(mCore.getOnlineStatus()!=USEROFFLINE && mCore.getOnlineStatus()!=USERTRYTOCONNECT){
		if(I2PStream_ID==0){
			mCore.createStreamObjectForUser(*newuser);
		}
	}
	emit signUserStatusChanged();
	return true;
}

bool CUserManager::checkIfUserExitsByI2PDestination(const QString I2PDestination)const{
	if(I2PDestination==mCore.getMyDestination()) return true;

	for(int i=0;i<mUsers.count();i++){
		if(mUsers.at(i)->getI2PDestination()==I2PDestination){
			return true;
		}
	}
	
	return false;
}
void CUserManager::changeUserPositionInUserList(int oldPos, int newPos)
{
	mUsers.swap(oldPos,newPos);
	saveUserList();
	emit signUserStatusChanged();
}

bool CUserManager::deleteUserByI2PDestination(QString I2PDestination){
	for(int i=0;i<mUsers.count();i++){
		if(mUsers.at(i)->getI2PDestination()==I2PDestination){
			if(mUsers.at(i)->getConnectionStatus()==ONLINE ||mUsers.at(i)->getConnectionStatus()==TRYTOCONNECT){
				mCore.deletePacketManagerByID(mUsers.at(i)->getI2PStreamID());
				mCore.getConnectionManager()->doDestroyStreamObjectByID(mUsers.at(i)->getI2PStreamID());
			}
			
			if(mCore.getConnectionManager()->isComponentStopped()==false){
				mCore.getConnectionManager()->doDestroyStreamObjectByID(mUsers.at(i)->getI2PStreamID());
			}
			mUsers.at(i)->deleteLater();
			mUsers.removeAt(i);
			saveUserList();
			emit signUserStatusChanged();
			return true;
		}
	}
	return false;
}

bool CUserManager::renameUserByI2PDestination(const QString Destination, const QString newNickname){
	for(int i=0;i<mUsers.size();i++){
		if(mUsers.at(i)->getI2PDestination()==Destination){
			mUsers.at(i)->setName(newNickname);
			saveUserList();
			emit signUserStatusChanged();
			return true;
		}
	}
	return false;
}

==> I2P-Messenger_0.2.19_Beta_src.zip: I2P-Messenger/src/UserManager.cpp <==
/*
    <one line to give the program's name and a brief idea of what it does.>
    Copyright (C) <year>  <name of author>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

*/

#include "UserManager.h"

#include "Protocol.h"
#include "UserBlockManager.h"
#include "Core.h"


CUserManager::CUserManager(CCore& Core, QString UserFileWithPath) 
:mCore(Core),mUserFileWithPath(UserFileWithPath)
{
  
}

CUserManager::~CUserManager()
{
      for(int i=0;i<this->mUsers.count();i++){
	    delete mUsers.at(i);
      }
      
      mUsers.clear();
}

void CUserManager::loadUserList(){
	QFile file(mUserFileWithPath);
     	if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
		return;
	
	QString NickName;
	QString I2PDest;
	
	QString line;
	QStringList temp;

	QByteArray bUserList=file.readAll();
        QTextStream in(bUserList);
	in.skipWhiteSpace();
	
     	while (!in.atEnd()) {
		line = in.readLine(550);
		temp=line.split("\t");
	
		if(temp[0]=="Nick:"){
			NickName=temp[1];
			}
		else if(temp[0]=="I2PDest:"){
			I2PDest=temp[1];
			this->addNewUser(NickName,I2PDest,false);
		}
		else if(temp[0]=="Invisible:"){
			if(temp[1]=="true"){
				getUserByI2P_Destination(I2PDest)->setInvisible(true);
			}
		}
		else if(temp[0]=="TorDest:"){
			//ignore it
		}
	}
	file.close();	
}

void CUserManager::saveUserList()const{
     QFile file(mCore.getConfigPath()+"/users.config");
     file.open(QIODevice::WriteOnly | QIODevice::Text);
     QTextStream out(&file);
     QString InvisibleText;

	for(int i=0;i<this->mUsers.count();i++){
		if(mUsers.at(i)->getIsInvisible()==true){
			InvisibleText="true";
		}
		else{
			InvisibleText="false";
		}

		out<<"Nick:\t"<<(mUsers.at(i)->getName())<<endl
		   <<"I2PDest:\t"<<(mUsers.at(i)->getI2PDestination())<<endl
		   <<"Invisible:\t"<<InvisibleText<<endl;
	}
	out.flush();
	file.close();
}

CUser* CUserManager::getUserByI2P_ID(qint32 ID)const{

	for(int i=0;i<mUsers.size();i++){
		if(mUsers.at(i)->getI2PStreamID()==ID){
			
			return mUsers.at(i);
		}
	}
	
	return NULL;
}
CUser* CUserManager::getUserByI2P_Destination(QString Destination)const{
	for(int i=0;i<mUsers.size();i++){
		if(mUsers.at(i)->getI2PDestination()==Destination){
			return mUsers.at(i);
		}
	}
	
	return NULL;
}


QString CUserManager::getUserInfosByI2P_Destination(QString Destination)const
{
	QString Infos="No Informations recived";

	for(int i=0;i<mUsers.size();i++){
		if(mUsers.at(i)->getI2PDestination()==Destination){
			CUser* theUser=mUsers.at(i);

			Infos= "Nickname:\t\t"  		+theUser->getName()+"\n";
			Infos+="Clientname:\t\t"		+theUser->getClientName()+"\n";
			Infos+="Clientversion:\t"		+theUser->getClientVersion()+"\n";
			Infos+="Protocolversion:\t"		+theUser->getProtocolVersion()+"\n";
			Infos+="Protocolversion min (Filetransfer):"+theUser->getMinProtocolVersionFiletransfer()+"\n";
			Infos+="Protocolversion max (Filetransfer):"+theUser->getMaxProtocolVersionFiletransfer()+"\n";
			
			if(theUser->getProtocolVersion_D()>=0.3){
				CRecivedInfos recivedInfos=theUser->getRecivedUserInfos();
				QString sAge;
				sAge.setNum(recivedInfos.Age,10);

				Infos+="\nRecived Userinfos:\n";
				Infos+="Nickname:\t\t"		+recivedInfos.Nickname+"\n";
				Infos+="Gender:\t\t"		+recivedInfos.Gender+"\n";
				Infos+="Age:\t\t"		+sAge+"\n";
				Infos+="Interests:\t\t"		+recivedInfos.Interests;
			}
		}
	}
	return Infos;
}

const QList<CUser*> CUserManager::getUserList()const{
	return mUsers;
}


bool CUserManager::addNewUser(QString Name,QString I2PDestination,qint32 I2PStream_ID,bool SaveUserList){
	CUserBlockManager& UserBlockManager=*(mCore.getUserBlockManager());
	CProtocol& Protocol=*(mCore.getProtocol());
      //The 52 character (or 60 with .b32.i2p 
      
      
	if(I2PDestination.length() != 516 && I2PDestination.length() != 60){
		qCritical()<<"CCore::addNewUser"
			   <<"\n\tDestination length must be 516(for B64) or 60(for B32)"
			   <<"\n\tlength : "<<I2PDestination.length()
			   <<"\n\taction add new User ignored";
		return false;
	}
	
	if(I2PDestination.length() == 516){
		if(!I2PDestination.right(4).contains("AAAA",Qt::CaseInsensitive)){
			qCritical()<<"\nCCore::addNewUser"
				  <<"\n\tthe last 4 char must be AAAA"
				  <<"\n\tDestination:\n"<<I2PDestination
				  <<"\n\taction add new User ignored";
			return false;
		}
	}else{
		if(!I2PDestination.right(8).contains(".b32.i2p",Qt::CaseInsensitive)){
			qCritical()<<"\nCCore::addNewUser"
				  <<"\n\tthe last 8 char must be .b32.i2p"
				  <<"\n\tDestination:\n"<<I2PDestination
				  <<"\n\taction add new User ignored";
			return false;
		}
	}
	
	

	if(UserBlockManager.isDestinationInBlockList(I2PDestination)==true){
		qCritical()<<"\nCCore::addNewUser"
			   <<"\n\tThe Destination: "<<I2PDestination
			   <<"\n\tis on the blocklist"
			   <<"\n\taction add new User ignored";
			   
		return false;
	}

	
	if(this->checkIfUserExitsByI2PDestination(I2PDestination)==true){
		qCritical()<<"\nCCore::addNewUser"
		    <<"\n\tthere is allready a user with this destination"
		    <<"\n\taction add new User ignored";

		return false;
	}

	if(I2PDestination==mCore.getMyDestination()){
		qCritical()<<"\nCCore::addNewUser"
		<<"\n\tIt's not allowed to add your self"
		<<"\n\taction add new User ignored";

		return false;
	}
	

	//add newuser
	CSoundManager& SoundManager=*(mCore.getSoundManager());
	CUser* newuser=new CUser(mCore, Protocol,Name,I2PDestination,I2PStream_ID);
	connect(newuser,SIGNAL(signNewIncomingMessageRecived()),&SoundManager,
		SLOT(slotNewChatMessage()));

	connect(newuser,SIGNAL(signConnectionOnline()),&SoundManager,
		SLOT(slotUserGoOnline()));

	connect(newuser,SIGNAL(signConnectionOffline()),&SoundManager,
		SLOT(slotUserGoOffline()));

	connect(newuser,SIGNAL(signOnlineStateChanged()),this,
		SIGNAL(signUserStatusChanged()));


	this->mUsers.append(newuser);
	if(SaveUserList==true){
		saveUserList();
	}

	if(mCore.getOnlineStatus()!=USEROFFLINE && mCore.getOnlineStatus()!=USERTRYTOCONNECT){
		if(I2PStream_ID==0){
			mCore.createStreamObjectForUser(*newuser);
		}
	}
	emit signUserStatusChanged();
	return true;
}

bool CUserManager::checkIfUserExitsByI2PDestination(const QString I2PDestination)const{
	if(I2PDestination==mCore.getMyDestination()) return true;

	for(int i=0;i<mUsers.count();i++){
		if(mUsers.at(i)->getI2PDestination()==I2PDestination){
			return true;
		}
	}
	
	return false;
}
void CUserManager::changeUserPositionInUserList(int oldPos, int newPos)
{
	mUsers.swap(oldPos,newPos);
	saveUserList();
	emit signUserStatusChanged();
}

bool CUserManager::deleteUserByI2PDestination(QString I2PDestination){
	for(int i=0;i<mUsers.count();i++){
		if(mUsers.at(i)->getI2PDestination()==I2PDestination){
			if(mUsers.at(i)->getConnectionStatus()==ONLINE ||mUsers.at(i)->getConnectionStatus()==TRYTOCONNECT){
				mCore.deletePacketManagerByID(mUsers.at(i)->getI2PStreamID());
				mCore.getConnectionManager()->doDestroyStreamObjectByID(mUsers.at(i)->getI2PStreamID());
			}
			
			if(mCore.getConnectionManager()->isComponentStopped()==false){
				mCore.getConnectionManager()->doDestroyStreamObjectByID(mUsers.at(i)->getI2PStreamID());
			}
			mUsers.at(i)->deleteLater();
			mUsers.removeAt(i);
			saveUserList();
			emit signUserStatusChanged();
			return true;
		}
	}
	return false;
}

bool CUserManager::renameUserByI2PDestination(const QString Destination, const QString newNickname){
	for(int i=0;i<mUsers.size();i++){
		if(mUsers.at(i)->getI2PDestination()==Destination){
			mUsers.at(i)->setName(newNickname);
			saveUserList();
			emit signUserStatusChanged();
			return true;
		}
	}
	return false;
}
